// 003 u_c_uses_list
// 24 dec 2001

// -- (C) John COLIBRI 2001
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_uses_list;
  interface
    uses Classes, u_c_basic_object;

    type c_use= // one "uses"
                Class(c_basic_object)
                  m_includes_path, m_unit_path: String;
                  m_level: Integer;
                  m_analyzed: Boolean;
                  m_count: Integer;

                  public
                    Constructor create_use(p_name, p_includes_path, p_unit_path: String; p_level: Integer); Virtual;
                    function f_display_use: String;
                    Destructor Destroy; Override;
                end;

         c_uses_list= // "uses" list
                 Class(c_basic_object)
                   public
                     m_includes_path: String;
                     m_c_uses_list: tStringList;

                     Constructor create_uses_list(p_name, p_includes_path: String); Virtual;

                     function f_use_count: Integer;
                     function f_c_use(p_use: Integer): c_use;
                     function f_c_use_by_name(p_name: String): c_use;

                     procedure add_use(p_use: String; p_c_use: c_use);
                     procedure add_new_use_or_increment_count(p_name, p_includes_path: String; p_level: Integer);
                     procedure update_uses_list_path(p_name, p_unit_path: String);

                     procedure display_uses_list;

                     Destructor Destroy; Override;
                   end;

  implementation
    uses SysUtils, u_c_display;

    // -- c_use

    Constructor c_use.create_use(p_name, p_includes_path, p_unit_path: String; p_level: Integer);
      begin
        Inherited create_basic_object(p_name);

        m_includes_path:= p_includes_path;
        m_unit_path:= p_unit_path;

        m_unit_path:= p_unit_path;
        m_level:= p_level;
      end; // create_use

    function c_use.f_display_use: String;
      begin
        Result:= Format('%-20s %2d %2d ', [m_name, m_level, m_count])+ LowerCase(m_unit_path)
      end; // f_display_use

    Destructor c_use.Destroy;
      begin
        InHerited;
      end; // Destroy

    // -- c_uses_list

    Constructor c_uses_list.create_uses_list(p_name, p_includes_path: String);
      begin
        Inherited create_basic_object(p_name);

        m_includes_path:= p_includes_path;
        m_c_uses_list:= tStringList.Create;
      end; // create_use_line

    function c_uses_list.f_use_count: Integer;
      begin
        Result:= m_c_uses_list.Count;
      end; // f_use_count

    function c_uses_list.f_c_use(p_use: Integer): c_use;
      begin
        Result:= c_use(m_c_uses_list.Objects[p_use]);
      end; //  f_c_use

    function c_uses_list.f_c_use_by_name(p_name: String): c_use;
      var l_index: Integer;
      begin
        l_index:= m_c_uses_list.IndexOf(p_name);
        if l_index>= 0
          then Result:= f_c_use(l_index)
          else Result:= Nil;
      end; // f_c_use_by_name

    procedure c_uses_list.add_use(p_use: String; p_c_use: c_use);
      begin
        m_c_uses_list.AddObject(p_use, p_c_use);
      end; // add_use

    procedure c_uses_list.add_new_use_or_increment_count(p_name, p_includes_path: String; p_level: Integer);
      var l_index: Integer;
      begin
        l_index:= m_c_uses_list.IndexOf(p_name);
        if l_index>= 0
          then Inc(f_c_use(l_index).m_count)
          Else add_use(p_name, c_use.create_use(p_name,
            p_includes_path, '', p_level));
      end; // add_new_use_or_increment_count

    procedure c_uses_list.update_uses_list_path(p_name, p_unit_path: String);
      var l_c_use: c_use;
      begin
        l_c_use:= f_c_use_by_name(p_name);
        if l_c_use= Nil
          then display_bug('update_uses_list_count could not find '+ p_name)
          else
            with l_c_use do
              if m_unit_path= ''
                then begin
                    m_unit_path:= p_unit_path;
                    m_analyzed:= true;
                  end
                else
                  if m_unit_path<> p_unit_path
                    then display_bug('not same unit '+ m_unit_path+ ' '+ p_unit_path);
      end; // update_uses_list_path

    procedure c_uses_list.display_uses_list;
      var l_use: Integer;
      begin
        display_line;
        display('uses_list '+ IntToStr(f_use_count));
        for l_use:= 0 to f_use_count- 1 do
          display(f_c_use(l_use).f_display_use);
      end; // display_uses_list

    Destructor c_uses_list.Destroy;
      var l_use: Integer;
      begin
        for l_use:= 0 to f_use_count- 1 do
          f_c_use(l_use).Free;
        m_c_uses_list.Free;
        Inherited;
      end; // Destroy

    begin // u_c_uses_list
    end. // u_c_uses_list
